package com.management.tzcontroller;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.management.constant.URLConstant;
import com.management.entity.Filter;
import com.management.utils.CrmRequestUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.client.RestClientException;
import org.springframework.web.client.RestTemplate;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

@RestController
@RequestMapping("/api/tz/yqht")
@Slf4j
public class YQHTController {

    private RestTemplate restTemplate = new RestTemplate();

    public static void main(String[] args) {
        new YQHTController().syncYQHT();
    }

    /**
     * 台账预签合同
     *
     * @return void
     * @Author weiloong_zhang
     */
    @PostMapping("/sync/yqht")
    public void syncYQHT() {
        CrmRequestUtil crmRequestUtil = new CrmRequestUtil();

        String[] syncStatus = {"option_pending_sync__c", "option_resync__c"};

        JSONObject getPreContract = crmRequestUtil.getCRMList(Arrays.asList(
                new Filter("GT", "last_modified_time", Arrays.asList("1748923200000")),
                new Filter("EQ", "life_status", Arrays.asList("normal")),
                //new Filter("EQ", "name", Arrays.asList("YQ2505260006")),
                new Filter("IN", "ledger_sync_status__c", Arrays.asList(syncStatus))
        ), "pre_signed_contract__c");

        JSONObject preContractRes = new JSONObject();

        try {
            preContractRes = restTemplate.postForObject(URLConstant.GET_CUSTOMIZE_LIST_URL, getPreContract, JSONObject.class);
        } catch (RestClientException e) {
            e.printStackTrace();
        }

        //判断预签合同是否存在
        if (!"success".equals(preContractRes.getString("errorDescription")) || preContractRes.getJSONObject("data").getJSONArray("dataList").isEmpty()) {
            log.info("预签合同不存在或者请求失败");
            return;
        }

        log.info("预签合同数据存在，将继续向下执行");

        for (Object preContractObj : preContractRes.getJSONObject("data").getJSONArray("dataList")) {
            JSONObject preContractData = JSON.parseObject(JSON.toJSONString(preContractObj));

            System.out.println("当前正在处理的数据为：" + preContractData);

            //开始截取关键数据
            String preContractId = preContractData.getString("_id");
            String preContractName = preContractData.getString("name");
            String modifyTime = preContractData.getString("last_modified_time");

            //查询日志，是否已经集成过了
            JSONObject isLog = new JSONObject();

            try {
                isLog = restTemplate.getForObject("http://localhost:18085/Log/query/log_data?table=send_log_tzht&log_type=TZYQHT&dataId=" + preContractId + "&mark=" + modifyTime, JSONObject.class);
            } catch (RestClientException e) {
                e.printStackTrace();
            }

            if (!isLog.getJSONArray("data").isEmpty()) {
                log.info("当前单据已经集成过了，将不再向下执行");
                continue;
            }

            log.info("当前单据没有集成过，将开始向下执行");

            //开始封装请求数据
            Map preContractReqMap = new HashMap<>();
            preContractReqMap.put("crmId", preContractId);
            preContractReqMap.put("contract", preContractName);//合同号
            preContractReqMap.put("contractName", preContractData.getString("project_name__c") != null ? preContractData.getString("project_name__c") : "");//合同名称
            preContractReqMap.put("signingUnit", preContractData.getString("contract_unit_name__c__r") != null ? preContractData.getString("contract_unit_name__c__r") : "");//签订单位（客户名）

            //开始获取地区
            //省
            String province = preContractData.getString("field_t3fgo__c__r") != null ? preContractData.getString("field_t3fgo__c__r") : "";
            //市
            String city = preContractData.getString("field_ngwth__c__r") != null ? preContractData.getString("field_ngwth__c__r") : "";
            //区
            String district = preContractData.getString("field_K5p2C__c__r") != null ? preContractData.getString("field_K5p2C__c__r") : "";
            //详细地址
            String address = preContractData.getString("field_740WI__c") != null ? preContractData.getString("field_740WI__c") : "";
            //地址
            String region = province + city + district + address;


            preContractReqMap.put("region", region);//地址
            preContractReqMap.put("isCompletion", 0);//todo 是否已完成(暂时默认0)

            //开始获取token
            String token = new LedgerTokenController().getToken();

            if (token == null || "".equals(token)) {
                log.info("台账token获取失败，将不再向下执行");
                continue;
            }

            //开始封装请求头
            HttpHeaders headers = new HttpHeaders();
            headers.set("Content-Type", "application/json");
            headers.set("accessToken", token);

            //开始封装请求实体
            HttpEntity preContractReqEntity = new HttpEntity(Arrays.asList(preContractReqMap), headers);

            System.out.println(JSON.parseObject(JSON.toJSONString(preContractReqEntity)));

            //开始发起请求
            String preContractUrl = "http://192.168.5.55:9011/crm/initContract";

            JSONObject preContractResult = new JSONObject();

            try {
                preContractResult = restTemplate.postForObject(preContractUrl, preContractReqEntity, JSONObject.class);
            } catch (RestClientException e) {
                e.printStackTrace();
            }

            System.out.println(JSON.parseObject(JSON.toJSONString(preContractResult)));

            //开始封装回写程序
            Map backReqMap = new HashMap();
            backReqMap.put("_id", preContractId);
            backReqMap.put("dataObjectApiName", "pre_signed_contract__c");

            //开始封装日志
            Map logMap = new HashMap<>();
            logMap.put("log_id", UUID.randomUUID().toString().replace("-", ""));
            logMap.put("log_type", "TZYQHT");
            logMap.put("syn_type", "0");
            logMap.put("data_name", preContractName);
            logMap.put("data_id", preContractId);
            logMap.put("mark", modifyTime);
            logMap.put("send_body", JSON.toJSONString(preContractReqEntity));
            logMap.put("send_res", JSON.toJSONString(preContractResult));
            logMap.put("tableName", "send_log_tzht");

            //判断是否执行成功
            if (preContractResult.getInteger("code") == 0) {
                //同步成功
                logMap.put("log_status", "0");
                logMap.put("res_body", "同步成功");
                backReqMap.put("ledger_sync_status__c", "option_sync_success__c");
                backReqMap.put("ledger_response_status__c", preContractResult.getString("msg"));
            } else {
                //同步失败
                logMap.put("log_status", "1");
                logMap.put("res_body", "同步失败：" + preContractResult.getString("msg"));
                backReqMap.put("ledger_sync_status__c", "option_sync_failure__c");
                backReqMap.put("ledger_response_status__c", preContractResult.getString("msg"));
            }

            //开始回写
            JSONObject backReq = crmRequestUtil.updateCRM(backReqMap);

            JSONObject backRes = new JSONObject();

            try {
                backRes = restTemplate.postForObject(URLConstant.UPDATE_CRM_CUSTOMIZE, backReq, JSONObject.class);
            } catch (RestClientException e) {
                e.printStackTrace();
            }

            System.out.println("回写结果为：" + backRes);

            //开始记录日志
            JSONObject logRes = new JSONObject();

            try {
                logRes = restTemplate.postForObject("http://localhost:18085/Log/insert/log_data", logMap, JSONObject.class);
            } catch (RestClientException e) {
                e.printStackTrace();
            }

            System.out.println("日志插入结果为：" + logRes);

        }
    }
}
